{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}
{																																							}
{	TktMultiBevel	v1.1 10/11/97																									}
{																																							}
{	The	TktMultiBevel	is an	enhanced Bevel with	the	possibility	to choose				}
{																																							}
{	-	BorderColor,																															}
{	-	BorderWidth,																															}
{	-	BevelColor,																																}
{	-	BevelWidth																																}
{	-	Shadow a.o.m.																															}
{																																							}
{	*	TktMultiBevel	is completely	FREEWARE.																			}
{	*	Do what	you	want with	it,	but	use	it at	your own	risk.										}
{																																							}
{	*	Comments,	suggestions	and	improvements are welcome.												}
{																																							}
{																																							}
{	For	Delphi 1 use the ressource file	*.d16, for Delphi	2	and	3	*.d32.				}
{	It's not necessary to	rename it.																						}
{																																							}
{																																							}
{	History:																																		}
{	v1.0 - 10/4/97 - Initial release																						}
{	v1.1 - 10/11/97	-	Added	Shadow properties, rebuilt procedures								}
{																																							}
{	e-mail:	MuK.Thaler@T-Online.de																							}
{	Kerstin	Thaler																															}
{																																							}
{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}

unit ktMBevel;

interface

uses
	{$IFDEF	Win32} Windows,	{$ELSE}	WinTypes,	WinProcs,	{$ENDIF} Classes,
	ExtCtrls,	Controls,	Graphics,	SysUtils;

type
	TBevelStyle	=	(bstLowered, bstNone,	bstRaised);
	TBevelShape	=	(bspBottomLine,	bspLeftLine, bspRect,	bspRightLine,	bspTopLine);
	TBevelWidth	=	1..MaxInt;
	TBorderWidth = 0..MaxInt;
	TShadowStyle = (ssBlack, ssCopy, ssMask, ssMaskNotPen, ssMerge,	ssMergeNotPen,
										ssNot, ssNotAND, ssNotCopy,	ssNotMask, ssNotMerge, ssNotOR,
										ssNotXOR,	ssTransparent, ssWhite,	ssXOR);

type
	TktMultiBevel	=	class(TGraphicControl)
	private
		FBevelInner: TBevelStyle;
		FBevelOuter: TBevelStyle;
		FBevelWidth: TBevelWidth;
		FBorderWidth:	TBorderWidth;
		FColor:	TColor;
		FColorFixed: Boolean;
		FColorHighlight: TColor;
		FColorShadow:	TColor;
		FShadowColor:	TColor;
		FShadowed: Boolean;
		FShadowOffsetX:	Integer;
		FShadowOffsetY:	Integer;
		FShadowStyle:	TShadowStyle;
		FShape:	TBevelShape;
		FTransparent:	Boolean;
		procedure	SetBevelInner(Value: TBevelStyle);
		procedure	SetBevelOuter(Value: TBevelStyle);
		procedure	SetBevelWidth(Value: TBevelWidth);
		procedure	SetBorderWidth(Value:	TBorderWidth);
		procedure	SetColor(Value:	TColor);
		procedure	SetColorHighlight(Value: TColor);
		procedure	SetColorFixed(Value: Boolean);
		procedure	SetColorShadow(Value:	TColor);
		procedure	SetShadowColor(Value:	TColor);
		procedure	SetShadowed(Value: Boolean);
		procedure	SetShadowOffsetX(Value:	Integer);
		procedure	SetShadowOffsetY(Value:	Integer);
		procedure	SetShadowStyle(Value:	TShadowStyle);
		procedure	SetShape(Value:	TBevelShape);
		procedure	SetTransparent(Value:	Boolean);
	protected

	public
		procedure	Paint; override;
		constructor	Create(AOwner: TComponent);	override;
		destructor Destroy;	override;
	published
		property Align;
		property BevelInner: TBevelStyle read	FBevelInner	write	SetBevelInner
								default	bstNone;
		property BevelOuter: TBevelStyle read	FBevelOuter	write	SetBevelOuter
								default	bstLowered;
		property BevelWidth: TBevelWidth read	FBevelWidth	write	SetBevelWidth	default	1;
		property BorderWidth:	TBorderWidth read	FBorderWidth write SetBorderWidth
								default	0;
		property Color:	TColor read	FColor write SetColor	default	clBtnFace;
		property ColorFixed: Boolean read	FColorFixed	write	SetColorFixed	default	True;
		property ColorHighlight: TColor	read FColorHighlight write SetColorHighlight
								default	clBtnHighlight;
		property ColorShadow:	TColor read	FColorShadow write SetColorShadow
								default	clBtnShadow;
		property ParentShowHint;
		property ShadowColor:	TColor read	FShadowColor write SetShadowColor
								default	clGray;
		property Shadowed: Boolean read	FShadowed	write	SetShadowed	default	False;
		property ShadowOffsetX:	Integer	read FShadowOffsetX	write	SetShadowOffsetX
								default	3;
		property ShadowOffsetY:	Integer	read FShadowOffsetY	write	SetShadowOffsetY
								default	3;
		property ShadowStyle:	TShadowStyle read	FShadowStyle write SetShadowStyle
								default	ssMerge;
		property Shape:	TBevelShape	read FShape	write	SetShape default bspRect;
		property ShowHint;
		property Transparent:	Boolean	read FTransparent	write	SetTransparent
								default	False;
		property Visible;
end;

procedure	Register;


implementation

{$IFDEF	Win32}
	{$R	*.d32}
{$ELSE}
	{$R	*.d16}
{$ENDIF}


procedure	Register;
begin
	RegisterComponents('Samples',[TktMultiBevel]);
end;

{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}

{	Utilities	}

function Min(X,	Y: Integer): Integer;
begin
	if X < Y then	Result :=	X	else Result	:= Y;
end;

function Max(X,	Y: Integer): Integer;
begin
	if X > Y then	Result :=	X	else Result	:= Y;
end;

function GetColorHighlight(BaseColor:	TColor): TColor;
begin
	Result :=	RGB(
		Min(GetRValue(ColorToRGB(BaseColor)) + 64, 255),
		Min(GetGValue(ColorToRGB(BaseColor)) + 64, 255),
		Min(GetBValue(ColorToRGB(BaseColor)) + 64, 255)
		);
end;

function GetColorShadow(BaseColor: TColor):	TColor;
begin
	Result :=	RGB(
		Max(GetRValue(ColorToRGB(BaseColor)) - 64, 0),
		Max(GetGValue(ColorToRGB(BaseColor)) - 64, 0),
		Max(GetBValue(ColorToRGB(BaseColor)) - 64, 0)
		);
end;

{	TktMultiBevel	}

constructor	TktMultiBevel.Create(AOwner: TComponent);
begin
	inherited	Create(AOwner);
		FBevelInner:=	bstNone;
		FBevelOuter:=	bstLowered;
		FBevelWidth:=	1;
		FBorderWidth:= 0;
		FColor:= clBtnFace;
		FColorFixed:=	True;
		FColorHighlight:=	clBtnHighlight;
		FColorShadow:= clBtnShadow;
		Height:= 50;
		FShadowColor:= clGray;
		FShadowed:=	False;
		FShadowOffsetX:= 3;
		FShadowOffsetY:= 3;
		FShadowStyle:= ssMerge;
		FShape:= bspRect;
		FTransparent:= False;
		Width:=	50;
end;

destructor TktMultiBevel.Destroy;
begin
	inherited	Destroy;
end;

procedure	TktMultiBevel.SetBevelInner(Value: TBevelStyle);
begin
	if Value <>	FBevelInner	then
	begin
		FBevelInner:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetBevelOuter(Value: TBevelStyle);
begin
	if Value <>	FBevelOuter	then
	begin
		FBevelOuter:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetBevelWidth(Value: TBevelWidth);
begin
	if Value <>	FBevelWidth	then
	begin
		FBevelWidth:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetBorderWidth(Value:	TBorderWidth);
begin
	if Value <>	FBorderWidth then
	begin
		FBorderWidth:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetColor(Value:	TColor);
begin
	if FColor	<> Value then	FColor:=	Value;
	if not FColorFixed then
	begin
		SetColorShadow(FColorShadow);
		SetColorHighlight(FColorHighlight);
	end
	else
	begin
		if FColor	=	clBtnFace	then
		begin
			FColorHighlight:=	clBtnHighlight;
			FColorShadow:= clBtnShadow;
		end
		else
		begin
			FColorHighlight:=	GetColorHighlight(FColor);
			FColorShadow:= GetColorShadow(FColor);
		end;
	end;
	Invalidate;
end;

procedure	TktMultiBevel.SetColorHighlight(Value: TColor);
begin
	if not FColorFixed then
	begin
		if Value <>	FColorHighlight	then
		begin
			FColorHighlight:=	Value;
			Invalidate;
		end;
	end;
end;

procedure	TktMultiBevel.SetColorShadow(Value:	TColor);
begin
	if not FColorFixed then
	begin
		if Value <>	FColorShadow then
		begin
			FColorShadow:=	Value;
			Invalidate;
		end;
	end;
end;

procedure	TktMultiBevel.SetColorFixed(Value: Boolean);
begin
	if Value <>	FColorFixed	then
	begin
		FColorFixed:=	Value;
		SetColor(FColor);
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShadowColor(Value:	TColor);
begin
	if Value <>	FShadowColor then
	begin
		FShadowColor:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShadowed(Value: Boolean);
begin
	if Value <>	FShadowed	then
	begin
		FShadowed:=	Value;
		if FShadowed then	FTransparent:= False;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShadowOffsetX(Value:	Integer);
begin
	if Value <>	FShadowOffsetX then
	begin
		FShadowOffsetX:=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShadowOffsetY(Value:	Integer);
begin
	if Value <>	FShadowOffsetY then
	begin
		FShadowOffsetY :=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShadowStyle(Value:	TShadowStyle);
begin
	if Value <>	FShadowStyle then
	begin
		FShadowStyle :=	Value;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.SetShape(Value:	TBevelShape);
begin
	if Value <>	FShape then
	begin
		FShape :=	Value;
		Invalidate;
	end;

end;

procedure	TktMultiBevel.SetTransparent(Value:	Boolean);
begin
	if Value <>	FTransparent then
	begin
		FTransparent :=	Value;
		if FTransparent	then FShadowed:= False;
		Invalidate;
	end;
end;

procedure	TktMultiBevel.Paint;
var
	Rc,	RectA: TRect;
	s, s1, s2, ox, oy:	Integer;
	P1,	P2,	P3,	P4,	P5,	P6,	P7,	P8:	TPoint;

	procedure	CalcShadow;
	begin
		case FShape	of
			bspTopLine		:	begin
												P1.x:= max(ox, 0);
												P1.y:= max(oy, 0);
												P2.x:= min(Rc.Right	-	1, Rc.Right	+	ox - 1);
												P2.y:= P1.y;
											end;
			bspBottomLine	:	begin
												P1.x:= max(ox, 0);
												P2.y:= min(Rc.Bottom + oy	-	1, Rc.Bottom - 1);
												P2.x:= min(Rc.Right	-	1, Rc.Right	+	ox - 1);
												P1.y:= P2.y;
											end;
			bspLeftLine		:	begin
												P1.x:= max(ox, 0);
												P1.y:= max(oy, 0);
												P2.x:= P1.x;
												P2.y:= min(Rc.Bottom - 1,	Rc.Bottom	+	oy - 1);
											end;
			bspRightLine	:	begin
												P2.x:= min(Rc.Right	-	1, Rc.Right	+	ox - 1);
												P1.y:= max(oy, 0);
												P1.x:= P2.x;
												P2.y:= min(Rc.Bottom + oy	-	1, Rc.Bottom - 1);
											end;
			bspRect				:	begin
												P1.x:= max(ox, 0);
												P1.y:= max(oy, 0);
												P2.x:= min(Rc.Right	-	1, Rc.Right	+	ox - 1);
												P2.y:= min(Rc.Bottom - 1,	Rc.Bottom	+	oy - 1);
											end;
		end;
		P3.x:= P2.x;
		P3.y:= P1.y;
		P4.x:= P1.x;
		P4.y:= P2.y;
		RectA:=	rect(P1.x, P1.y, P2.x, P2.y);
	end;

	procedure	PaintShadow;
	var
		i: Integer;
	begin
		with Canvas	do
		begin
			Pen.Width:=	1;
			Pen.Color:=	FShadowColor;
			case FShadowStyle	of
				ssBlack				:	Pen.Mode:= pmBlack;
				ssCopy				:	Pen.Mode:= pmCopy;
				ssMask				:	Pen.Mode:= pmMask;
				ssMaskNotPen	:	Pen.Mode:= pmMaskNotPen;
				ssMerge				:	Pen.Mode:= pmMerge;
				ssMergeNotPen	:	Pen.Mode:= pmMergeNotPen;
				ssNot					:	Pen.Mode:= pmNot;
				ssNotAND			:	Pen.Mode:= pmMaskPenNot;
				ssNotCopy			:	Pen.Mode:= pmNotCopy;
				ssNotMask			:	Pen.Mode:= pmNotMask;
				ssNotMerge		:	Pen.Mode:= pmNotMerge;
				ssNotOR				:	Pen.Mode:= pmMergePenNot;
				ssNotXOR			:	Pen.Mode:= pmNotXor;
				ssTransparent	:	Pen.Mode:= pmNop;
				ssWhite				:	Pen.Mode:= pmWhite;
				ssXOR					:	Pen.Mode:= pmXor;
			end;
			if Shape = bspRect then
			begin
				for	i:=	1	to s do
				begin
					PolyLine([P1,	P3,	P2,	P4,	P1]);
					inc(P1.x);
					inc(P1.y);
					dec(P2.x);
					dec(P2.y);
					P3.x:= P2.x;
					P3.y:= P1.y;
					P4.x:= P1.x;
					P4.y:= P2.y;
				end;
			end
			else
			begin
				for	i:=	1	to s do
				begin
					MoveTo(P1.x, p1.y);
					LineTo(P2.x, P2.y);
					case Shape of
						bspTopLine		:	begin
															inc(P1.y);
															inc(P2.y);
														end;
						bspBottomLine	:	begin
															dec(P1.y);
															dec(P2.y);
														end;
						bspLeftLine		:	begin
															inc(P1.x);
															inc(P2.x);
														end;
						bspRightLine	:	begin
															dec(P1.x);
															dec(P2.x);
														end;
					end;
				end;
			end;
			Pen.Mode:= pmCopy;
		end;
	end;

	procedure	CalcBevelOuter;
	begin
		case FShape	of
			bspTopLine		:	begin
												P1.x:= max(-ox,	0);
												P1.y:= max(-oy,	0);
												P2.x:= min(Rc.Right, Rc.Right	-	ox);
												P2.y:= P1.y	+	s;
											end;
			bspBottomLine	:	begin
												P1.x:= max(-ox,	0);
												P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
												P2.x:= min(Rc.Right, Rc.Right	-	ox);
												P1.y:= P2.y	-	s;
											end;
			bspLeftLine		:	begin
												P1.x:= max(-ox,	0);
												P1.y:= max(-oy,	0);
												P2.x:= P1.x	+	s;
												P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
											end;
			bspRightLine	:	begin
												P2.x:= min(Rc.Right, Rc.Right	-	ox);
												P1.y:= max(-oy,	0);
												P1.x:= P2.x	-	s;
												P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
											end;
			bspRect				:	begin
												P1.x:= max(-ox,	0);
												P1.y:= max(-oy,	0);
												P2.x:= min(Rc.Right, Rc.Right	-	ox);
												P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
											end;
		end;
		RectA:=	rect(P1.x, P1.y, P2.x, P2.y);
	end;

	procedure	PaintBevelOuter;
	begin
		with Canvas, RectA do
		begin
			case FBevelOuter of
				bstLowered:	Frame3D(Canvas,RectA,FColorShadow,FColorHighlight,FBevelWidth);
				bstRaised	:	Frame3D(Canvas,RectA,FColorHighlight,FColorShadow,FBevelWidth);
			end;
		end;
	end;

	procedure	CalcBorder;
	var	v: Integer;
	begin
		if FBevelOuter <>	bstNone	then
		begin
			case FShape	of
				bspTopLine		:	begin
													P1.x:= max(FBevelWidth - ox, FBevelWidth);
													P1.y:= max(FBevelWidth - oy, FBevelWidth);
													P2.x:= min(Rc.Right	-	FBevelWidth, Rc.Right	-	ox
																		-	FBevelWidth);
													P2.y:= P1.y	+	FBorderwidth;
												end;
				bspBottomLine	:	begin
													P1.x:= max(FBevelWidth - ox, FBevelWidth);
													P2.y:= min(Rc.Bottom - FBevelWidth,	Rc.Bottom	-	oy
																		-	FBevelWidth);
													P2.x:= min(Rc.Right	-	FBevelWidth, Rc.Right	-	ox
																		-	FBevelWidth);
													P1.y:= P2.y	-	FBorderwidth;
												end;
				bspLeftLine		:	begin
													P1.x:= max(FBevelWidth - ox, FBevelWidth);
													P1.y:= max(FBevelWidth - oy, FBevelWidth);
													P2.x:= P1.x	+	FBorderWidth;
													P2.y:= min(Rc.Bottom - FBevelWidth,	Rc.Bottom	-	oy
																	-	FBevelWidth);
												end;
				bspRightLine	:	begin
													P2.x:= min(Rc.Right	-	FBevelWidth, Rc.Right	-	ox
																	-	FBevelWidth);
													P1.y:= max(	FBevelWidth	-	oy,	FBevelWidth);
													P1.x:= P2.x	-	FBorderWidth;
													P2.y:= min(Rc.Bottom - FBevelWidth,	Rc.Bottom	-	oy
																	-	FBevelWidth);
												end;
				bspRect				:	begin
												P1.x:= max(	FBevelWidth	-	ox,	FBevelWidth);
												P1.y:= max(	FBevelWidth	-	oy,	FBevelWidth);
												P2.x:= min(Rc.Right	-	FBevelWidth, Rc.Right	-	ox
																	-	FBevelWidth);
												P2.y:= min(Rc.Bottom - FBevelWidth,	Rc.Bottom	-	oy
																	-	FBevelWidth);
											end;
			end;
		end
		else
		begin
			case FShape	of
				bspTopLine		:	begin
													P1.x:= max(- ox, 0);
													P1.y:= max(- oy, 0);
													P2.x:= min(Rc.Right, Rc.Right	-	ox);
													P2.y:= P1.y	+	FBorderwidth;
												end;
				bspBottomLine	:	begin
													P1.x:= max(- ox, 0);
													P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
													P2.x:= min(Rc.Right, Rc.Right	-	ox);
													P1.y:= P2.y	-	FBorderwidth;
												end;
				bspLeftLine		:	begin
													P1.x:= max(- ox, 0);
													P1.y:= max(- oy, 0);
													P2.x:= P1.x	+	FBorderWidth;
													P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
												end;
				bspRightLine	:	begin
													P2.x:= min(Rc.Right, Rc.Right	-	ox);
													P1.y:= max(- oy, 0);
													P1.x:= P2.x	-	FBorderWidth;
													P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
												end;
				bspRect				:	begin
													P1.x:= max(-ox,	0);
													P1.y:= max(-oy,	0);
													P2.x:= min(Rc.Right, Rc.Right	-	ox);
													P2.y:= min(Rc.Bottom,	Rc.Bottom	-	oy);
												end;
			end;
		end;
		RectA:=	rect(P1.x, P1.y, P2.x, P2.y);
	end;

	procedure	PaintBorder;
	var
		i: Integer;
	begin
		with Canvas, RectA do
		begin
			Brush.Color:=	FColor;
			for	i:=1 to	FBorderWidth do
			begin
				FrameRect(RectA);
				inc(P1.x);
				inc(P1.y);
				dec(P2.x);
				dec(P2.y);
				RectA:=	rect(P1.x, P1.y, P2.x, P2.y);
			end;
		end;
	end;

	procedure	CalcBevelInner;
	begin
		P1.x:= max(s1	-	ox,	s1);
		P1.y:= max(s1	-	oy,	s1);
		P2.x:= min(Rc.Right	-	s1,	Rc.Right - ox	-	s1);
		P2.y:= min(Rc.Bottom - s1, Rc.Bottom - oy	-	s1);
		RectA:=	rect(P1.x, P1.y, P2.x, P2.y);
	end;

	procedure	PaintBevelInner;
	begin
		with Canvas, RectA do
		begin
			case FBevelInner of
				bstLowered:	Frame3D(Canvas,RectA,FColorShadow,FColorHighlight,FBevelWidth);
				bstRaised	:	Frame3D(Canvas,RectA,FColorHighlight,FColorShadow,FBevelWidth);
			end;
		end;
	end;

	procedure	BevelRect;
	begin
		if (FBevelInner	<> bstNone)	and	(FShape	=	bspRect) then	s2:= FBevelWidth
			else s2:=	0;
		if FBevelOuter = bstNone then	s1:= FBorderWidth
			else
			if FShape	=	bspRect	then s1:=	FBevelWidth	+	FBorderWidth
				else s1:=	2*FBevelWidth	+	FBorderWidth;
		s:=	s1 + s2;
		if not FShadowed then
		begin
			ox:= 0;
			oy:= 0;
		end
		else
		begin
			if s > 0 then
			begin
				ox:= FShadowOffsetX;
				oy:= FShadowOffsetY;
				CalcShadow;
				PaintShadow;
			end;
		end;
		if FBevelOuter <>	bstNone	then
		begin
			CalcBevelOuter;
			PaintBevelOuter;
		end;
		if (FBorderWidth > 0)	and	not	FTransparent then
		begin
			CalcBorder;
			PaintBorder;
		end;
		if (FBevelInner	<> bstNone)	and	(FShape	=	bspRect) then
		begin
			CalcBevelInner;
			PaintBevelInner;
		end;
	end;

begin
	Rc:= GetClientRect;
	BevelRect;
end;

end.



